////////////////////////////////////////////////////////////////////////
// Class:       LArPIDInterfaceTest
// Plugin Type: analyzer (art v3_01_02)
// File:        LArPIDInterfaceTest_module.cc
//
// Generated at Wed Jan 22 08:22:00 2025 by Matthew Rosenberg using cetskelgen
// from cetlib version v3_05_01.
////////////////////////////////////////////////////////////////////////

//These headers MUST come before any ROOT includes to deal with ROOT/libtorch conflict
#include "LArPIDInterface.h"
#include "LArCVBackTracker.h"

#include "art/Framework/Core/EDAnalyzer.h"
#include "art/Framework/Core/ModuleMacros.h"
#include "art/Framework/Principal/Event.h"
#include "art/Framework/Principal/Handle.h"
#include "art/Framework/Principal/Run.h"
#include "art/Framework/Principal/SubRun.h"
#include "art_root_io/TFileService.h"
#include "canvas/Utilities/InputTag.h"
#include "fhiclcpp/ParameterSet.h"
#include "messagefacility/MessageLogger/MessageLogger.h"

#include "ubobj/WcpPort/SpacePointStructs.h"
#include "nusimdata/SimulationBase/MCParticle.h"
#include "larlite/DataFormat/storage_manager.h"

#include <TTree.h>

#include <unordered_map>
#include <cmath>
#include <cstdlib>

class LArPIDInterfaceTest;

class LArPIDInterfaceTest : public art::EDAnalyzer {
public:
  explicit LArPIDInterfaceTest(fhicl::ParameterSet const& p);
  // The compiler-generated destructor is fine for non-base
  // classes without bare pointers or other resource use.

  // Plugins should not be copied or assigned.
  LArPIDInterfaceTest(LArPIDInterfaceTest const&) = delete;
  LArPIDInterfaceTest(LArPIDInterfaceTest&&) = delete;
  LArPIDInterfaceTest& operator=(LArPIDInterfaceTest const&) = delete;
  LArPIDInterfaceTest& operator=(LArPIDInterfaceTest&&) = delete;

  // Required functions.
  void analyze(art::Event const& e) override;

  void beginJob() override;
  void endJob() override;

private:

  // Declare member data here.
  std::string fLArCVImageFile;
  std::string fModel;
  //bool fUseGPU;
  unsigned int fPixelThreshold; // only run model over prongs with at least this many pixels in at least one plane
  bool fAllPlaneThreshold; //if true, only run model over prongs that pass the pixel threshold in all three planes
  bool fTickBack; //if true, read in larcv images with reverse time order
  bool fDebug; //if true, print tensor values and info for debugging LArPID interface
  unsigned int iImg_start;

  //storage managers for accessing info from merged_dlreco files
  larcv::IOManager* iolcv;
  larlite::storage_manager* ioll;

  //LArPID torchscript model
  LArPID::TorchModel model;

  // Output TTree variables
  TTree* tree_;
  int run_;
  int subrun_;
  int event_;
  int prong_tid_;
  int wirecell_pid_;
  int larpid_pid_;
  int larpid_process_;
  float larpid_completeness_;
  float larpid_purity_;
  float larpid_pidScore_el_;
  float larpid_pidScore_ph_;
  float larpid_pidScore_mu_;
  float larpid_pidScore_pi_;
  float larpid_pidScore_pr_;
  float larpid_procScore_prim_;
  float larpid_procScore_ntrl_;
  float larpid_procScore_chgd_;
  int truthMatch_pid_;
  int truthMatch_tid_;
  float truthMatch_purity_;
  float truthMatch_completeness_;
  int truthMatch_nSimParts_;
  int truthMatch_simPart_pid_[1000];
  float truthMatch_simPart_purity_[1000];
  int prongImg_plane0_nPix_;
  int prongImg_plane0_row_[262144];
  int prongImg_plane0_col_[262144];
  float prongImg_plane0_val_[262144];
  int prongImg_plane1_nPix_;
  int prongImg_plane1_row_[262144];
  int prongImg_plane1_col_[262144];
  float prongImg_plane1_val_[262144];
  int prongImg_plane2_nPix_;
  int prongImg_plane2_row_[262144];
  int prongImg_plane2_col_[262144];
  float prongImg_plane2_val_[262144];
  int contextImg_plane0_nPix_;
  int contextImg_plane0_row_[262144];
  int contextImg_plane0_col_[262144];
  float contextImg_plane0_val_[262144];
  int contextImg_plane1_nPix_;
  int contextImg_plane1_row_[262144];
  int contextImg_plane1_col_[262144];
  float contextImg_plane1_val_[262144];
  int contextImg_plane2_nPix_;
  int contextImg_plane2_row_[262144];
  int contextImg_plane2_col_[262144];
  float contextImg_plane2_val_[262144];
  int n_spacePoints_;
  float spacePoint_x_[100000];
  float spacePoint_y_[100000];
  float spacePoint_z_[100000];
  int spacePoint_tick_[100000];
  int spacePoint_p0wire_[100000];
  int spacePoint_p1wire_[100000];
  int spacePoint_p2wire_[100000];
  float cropPoint_x_;
  float cropPoint_y_;
  float cropPoint_z_;

};


void LArPIDInterfaceTest::beginJob() {

  iImg_start = 0;

  auto tickDir = larcv::IOManager::kTickForward;
  if(fTickBack) tickDir = larcv::IOManager::kTickBackward;
  iolcv = new larcv::IOManager(larcv::IOManager::kREAD,"larcv",tickDir);
  iolcv -> add_in_file(fLArCVImageFile);
  if(fTickBack) iolcv -> reverse_all_products();
  iolcv -> initialize();

  ioll = new larlite::storage_manager(larlite::storage_manager::kREAD);
  ioll -> add_in_filename(fLArCVImageFile);
  ioll -> set_data_to_read( "mctrack", "mcreco" );
  ioll -> set_data_to_read( "mcshower", "mcreco" );
  ioll -> set_data_to_read( "mctruth", "generator" );
  ioll -> set_data_to_read( "gtruth", "generator" );
  ioll -> set_data_to_read( "mcflux", "generator" );
  ioll -> set_data_to_read( "mctruth", "corsika" );
  ioll -> open();

  const char* ubdlDir = std::getenv("UBDL_BASEDIR");
  std::string modelPath(ubdlDir);
  modelPath += "/model_weights/";
  modelPath += fModel;
  model.Initialize(modelPath, fDebug);

}


LArPIDInterfaceTest::LArPIDInterfaceTest(fhicl::ParameterSet const& p)
  : EDAnalyzer{p},
     fLArCVImageFile(p.get<std::string>("LArCVImageFile")),
     fModel(p.get<std::string>("Model")),
     //fUseGPU(p.get<bool>("UseGPU")),
     fPixelThreshold(p.get<unsigned int>("PixelThreshold")),
     fAllPlaneThreshold(p.get<bool>("AllPlaneThreshold")),
     fTickBack(p.get<bool>("TickBack")),
     fDebug(p.get<bool>("Debug"))
  // More initializers here.
{
  // Call appropriate consumes<>() for any products to be retrieved by this module.
  art::ServiceHandle<art::TFileService> fileService;
  tree_ = fileService->make<TTree>("ProngTree", "ProngTree");
  tree_->Branch("run", &run_, "run/I");
  tree_->Branch("subrun", &subrun_, "subrun/I");
  tree_->Branch("event", &event_, "event/I");
  tree_->Branch("prong_tid", &prong_tid_, "prong_tid/I");
  tree_->Branch("wirecell_pid", &wirecell_pid_, "wirecell_pid/I");
  tree_->Branch("larpid_pid", &larpid_pid_, "larpid_pid/I");
  tree_->Branch("larpid_process", &larpid_process_, "larpid_process/I");
  tree_->Branch("larpid_completeness", &larpid_completeness_, "larpid_completeness/F");
  tree_->Branch("larpid_purity", &larpid_purity_, "larpid_purity/F");
  tree_->Branch("larpid_pidScore_el", &larpid_pidScore_el_, "larpid_pidScore_el/F");
  tree_->Branch("larpid_pidScore_ph", &larpid_pidScore_ph_, "larpid_pidScore_ph/F");
  tree_->Branch("larpid_pidScore_mu", &larpid_pidScore_mu_, "larpid_pidScore_mu/F");
  tree_->Branch("larpid_pidScore_pi", &larpid_pidScore_pi_, "larpid_pidScore_pi/F");
  tree_->Branch("larpid_pidScore_pr", &larpid_pidScore_pr_, "larpid_pidScore_pr/F");
  tree_->Branch("larpid_procScore_prim", &larpid_procScore_prim_, "larpid_procScore_prim/F");
  tree_->Branch("larpid_procScore_ntrl", &larpid_procScore_ntrl_, "larpid_procScore_ntrl/F");
  tree_->Branch("larpid_procScore_chgd", &larpid_procScore_chgd_, "larpid_procScore_chgd/F");
  tree_->Branch("truthMatch_pid", &truthMatch_pid_, "truthMatch_pid/I");
  tree_->Branch("truthMatch_tid", &truthMatch_tid_, "truthMatch_tid/I");
  tree_->Branch("truthMatch_purity", &truthMatch_purity_, "truthMatch_purity/F");
  tree_->Branch("truthMatch_completeness", &truthMatch_completeness_, "truthMatch_completeness/F");
  tree_->Branch("truthMatch_nSimParts", &truthMatch_nSimParts_, "truthMatch_nSimParts/I");
  tree_->Branch("truthMatch_simPart_pid", truthMatch_simPart_pid_, "truthMatch_simPart_pid[truthMatch_nSimParts]/I");
  tree_->Branch("truthMatch_simPart_purity", truthMatch_simPart_purity_, "truthMatch_simPart_purity[truthMatch_nSimParts]/F");
  tree_->Branch("prongImg_plane0_nPix", &prongImg_plane0_nPix_, "prongImg_plane0_nPix/I");
  tree_->Branch("prongImg_plane0_row", prongImg_plane0_row_, "prongImg_plane0_row[prongImg_plane0_nPix]/I");
  tree_->Branch("prongImg_plane0_col", prongImg_plane0_col_, "prongImg_plane0_col[prongImg_plane0_nPix]/I");
  tree_->Branch("prongImg_plane0_val", prongImg_plane0_val_, "prongImg_plane0_val[prongImg_plane0_nPix]/F");
  tree_->Branch("prongImg_plane1_nPix", &prongImg_plane1_nPix_, "prongImg_plane1_nPix/I");
  tree_->Branch("prongImg_plane1_row", prongImg_plane1_row_, "prongImg_plane1_row[prongImg_plane1_nPix]/I");
  tree_->Branch("prongImg_plane1_col", prongImg_plane1_col_, "prongImg_plane1_col[prongImg_plane1_nPix]/I");
  tree_->Branch("prongImg_plane1_val", prongImg_plane1_val_, "prongImg_plane1_val[prongImg_plane1_nPix]/F");
  tree_->Branch("prongImg_plane2_nPix", &prongImg_plane2_nPix_, "prongImg_plane2_nPix/I");
  tree_->Branch("prongImg_plane2_row", prongImg_plane2_row_, "prongImg_plane2_row[prongImg_plane2_nPix]/I");
  tree_->Branch("prongImg_plane2_col", prongImg_plane2_col_, "prongImg_plane2_col[prongImg_plane2_nPix]/I");
  tree_->Branch("prongImg_plane2_val", prongImg_plane2_val_, "prongImg_plane2_val[prongImg_plane2_nPix]/F");
  tree_->Branch("contextImg_plane0_nPix", &contextImg_plane0_nPix_, "contextImg_plane0_nPix/I");
  tree_->Branch("contextImg_plane0_row", contextImg_plane0_row_, "contextImg_plane0_row[contextImg_plane0_nPix]/I");
  tree_->Branch("contextImg_plane0_col", contextImg_plane0_col_, "contextImg_plane0_col[contextImg_plane0_nPix]/I");
  tree_->Branch("contextImg_plane0_val", contextImg_plane0_val_, "contextImg_plane0_val[contextImg_plane0_nPix]/F");
  tree_->Branch("contextImg_plane1_nPix", &contextImg_plane1_nPix_, "contextImg_plane1_nPix/I");
  tree_->Branch("contextImg_plane1_row", contextImg_plane1_row_, "contextImg_plane1_row[contextImg_plane1_nPix]/I");
  tree_->Branch("contextImg_plane1_col", contextImg_plane1_col_, "contextImg_plane1_col[contextImg_plane1_nPix]/I");
  tree_->Branch("contextImg_plane1_val", contextImg_plane1_val_, "contextImg_plane1_val[contextImg_plane1_nPix]/F");
  tree_->Branch("contextImg_plane2_nPix", &contextImg_plane2_nPix_, "contextImg_plane2_nPix/I");
  tree_->Branch("contextImg_plane2_row", contextImg_plane2_row_, "contextImg_plane2_row[contextImg_plane2_nPix]/I");
  tree_->Branch("contextImg_plane2_col", contextImg_plane2_col_, "contextImg_plane2_col[contextImg_plane2_nPix]/I");
  tree_->Branch("contextImg_plane2_val", contextImg_plane2_val_, "contextImg_plane2_val[contextImg_plane2_nPix]/F");
  tree_->Branch("n_spacePoints", &n_spacePoints_, "n_spacePoints/I");
  tree_->Branch("spacePoint_x", spacePoint_x_, "spacePoint_x[n_spacePoints]/F");
  tree_->Branch("spacePoint_y", spacePoint_y_, "spacePoint_y[n_spacePoints]/F");
  tree_->Branch("spacePoint_z", spacePoint_z_, "spacePoint_z[n_spacePoints]/F");
  tree_->Branch("spacePoint_tick", spacePoint_tick_, "spacePoint_tick[n_spacePoints]/I");
  tree_->Branch("spacePoint_p0wire", spacePoint_p0wire_, "spacePoint_p0wire[n_spacePoints]/I");
  tree_->Branch("spacePoint_p1wire", spacePoint_p1wire_, "spacePoint_p1wire[n_spacePoints]/I");
  tree_->Branch("spacePoint_p2wire", spacePoint_p2wire_, "spacePoint_p2wire[n_spacePoints]/I");
  tree_->Branch("cropPoint_x", &cropPoint_x_, "cropPoint_x/F");
  tree_->Branch("cropPoint_y", &cropPoint_y_, "cropPoint_y/F");
  tree_->Branch("cropPoint_z", &cropPoint_z_, "cropPoint_z/F");
}

void LArPIDInterfaceTest::analyze(art::Event const& e)
{

  std::cout << "NEW EVENT: run "<<e.id().run()<<", subrun "<<e.id().subRun()<<", event "<<e.id().event() << std::endl;

  run_ = e.id().run();
  subrun_ = e.id().subRun();
  event_ = e.id().event();

  larcv::EventImage2D* wireImage2D = nullptr;
  larcv::EventImage2D* cosmicImage2D = nullptr;
  for (unsigned int iImg = iImg_start; iImg < iolcv->get_n_entries(); ++iImg){
    iolcv -> read_entry(iImg);
    larcv::EventImage2D* wireImg = (larcv::EventImage2D*)(iolcv->get_data(larcv::kProductImage2D,"wire"));
    if(wireImg->run() == e.id().run() && wireImg->subrun() == e.id().subRun() && wireImg->event() == e.id().event()){
      wireImage2D = wireImg;
      cosmicImage2D = (larcv::EventImage2D*)(iolcv->get_data(larcv::kProductImage2D,"thrumu"));
      iImg_start = iImg+1;
      ioll -> go_to(iImg);
      break;
    }
  }

  if(wireImage2D == nullptr || cosmicImage2D == nullptr){
    std::cout << "MATCHING EVENT FROM LARCV IMAGE FILE NOT FOUND!!! Skipping event..." << std::endl;
    return;
  }
  else{
    std::cout << "Found matching event from larcv image file. We have the images!" << std::endl;
  }

  std::vector<larcv::Image2D> adc_v = wireImage2D->Image2DArray();
  std::vector<larcv::Image2D> thrumu_v = cosmicImage2D->Image2DArray();

  std::unordered_map<int, LArPID::ParticleInfo> particleMap;

  auto const& particles = *e.getValidHandle<std::vector<simb::MCParticle>>("wirecellPF");
  for (auto const& particle : particles) {
    std::cout << "reco particle: "
              << "trackID = " << particle.TrackId() << ", "
              << "pdg = " << particle.PdgCode() << ", "
              << "start position = ("<<particle.Vx()<<","<<particle.Vy()<<","<<particle.Vz()<<"), "
              << "end position = ("<<particle.EndX()<<","<<particle.EndY()<<","<<particle.EndZ()<<")"
              << std::endl;
    if(std::abs(particle.PdgCode()) == 11 || std::abs(particle.PdgCode()) == 22){
      TVector3 crop(particle.Vx(), particle.Vy(), particle.Vz());
      LArPID::ParticleInfo newParticle(crop);
      particleMap[particle.TrackId()] = newParticle;
    }
    else{
      TVector3 crop(particle.EndX(), particle.EndY(), particle.EndZ());
      LArPID::ParticleInfo newParticle(crop);
      particleMap[particle.TrackId()] = newParticle;
    }
  }

  auto const& spacePoints = *e.getValidHandle<std::vector<TrecSpacePoint>>("portedWCSpacePointsTrec");
  for (auto const& point : spacePoints) {
    if(particleMap.count(point.real_cluster_id) < 1) continue;
    larlite::larflow3dhit hit;
    hit.clear(); hit.reserve(3); //not needed for LArPID
    hit.push_back(point.x); hit.push_back(point.y); hit.push_back(point.z); //not needed for LArPID
    for(unsigned int p = 0; p < adc_v.size(); ++p){
      auto center2D = larutil::GeometryHelper::GetME()->Point_3Dto2D(point.x,point.y,point.z,p);
      if(p == 0) hit.tick = (int)(center2D.t/larutil::GeometryHelper::GetME()->TimeToCm() + 3200.);
      if(p < hit.targetwire.size()) hit.targetwire[p] = (int)(center2D.w/larutil::GeometryHelper::GetME()->WireToCm());
      else hit.targetwire.push_back((int)(center2D.w/larutil::GeometryHelper::GetME()->WireToCm()));
    }
    particleMap[point.real_cluster_id].larflowProng.push_back(hit);
    /*std::cout << "TrecSpacePoint: "
              << "x = " << point.x << ", "
              << "y = " << point.y << ", "
              << "z = " << point.z << ", "
              << "real_cluster_id = " << point.real_cluster_id
              << std::endl;*/
  }


  for (const auto& partMapPair : particleMap){

    auto prong_vv = LArPID::make_cropped_initial_sparse_prong_image_reco(adc_v, thrumu_v,
     partMapPair.second.larflowProng, partMapPair.second.cropPoint, 10., 512, 512);
    std::cout << "successfully made prong image for trackID "<<partMapPair.first<<" with "<<prong_vv[0].size()<<" plane 0 pixels, "<<prong_vv[1].size()<<" plane 1 pixels, and "<<prong_vv[2].size()<<" plane 2 pixels!" << std::endl;

    bool thresholdPassInOne = false;
    bool thresholdPassInAll = true;
    for(size_t p = 0; p < 3; ++p){
      if(prong_vv[p].size() >= fPixelThreshold) thresholdPassInOne = true;
      else thresholdPassInAll = false;
    }

    if(thresholdPassInAll || (thresholdPassInOne && !fAllPlaneThreshold)){

      LArPID::ModelOutput output = model.run_inference(prong_vv);
      std::cout << "Successfuly ran LArPID! Model outputs:" << std::endl;
      std::cout << "    PID: " << output.pid << std::endl;
      std::cout << "    Production process: " << output.process << std::endl;
      std::cout << "    completeness: " << output.completeness << std::endl;
      std::cout << "    purity: " << output.purity << std::endl;
      std::cout << "    electron_score: " << output.electron_score << std::endl;
      std::cout << "    photon_score: " << output.photon_score << std::endl;
      std::cout << "    muon_score: " << output.muon_score << std::endl;
      std::cout << "    pion_score: " << output.pion_score << std::endl;
      std::cout << "    proton_score: " << output.proton_score << std::endl;
      std::cout << "    primary_score: " << output.primary_score << std::endl;
      std::cout << "    neutralParent_score: " << output.neutralParent_score << std::endl;
      std::cout << "    chargedParent_score: " << output.chargedParent_score << std::endl;

      prong_tid_ = partMapPair.first;

      wirecell_pid_ = 0;
      for (auto const& particle : particles) {
        if(particle.TrackId() == partMapPair.first){
          wirecell_pid_ = particle.PdgCode();
          break;
        }
      }

      larpid_pid_ = output.pid;
      larpid_process_ = output.process;
      larpid_completeness_ = output.completeness;
      larpid_purity_ = output.purity;
      larpid_pidScore_el_ = output.electron_score;
      larpid_pidScore_ph_ = output.photon_score;
      larpid_pidScore_mu_ = output.muon_score;
      larpid_pidScore_pi_ = output.pion_score;
      larpid_pidScore_pr_ = output.proton_score;
      larpid_procScore_prim_ = output.primary_score;
      larpid_procScore_ntrl_ = output.neutralParent_score;
      larpid_procScore_chgd_ = output.chargedParent_score;

      LArCVBackTrack::TruthMatchResults truthMatch = LArCVBackTrack::run_backtracker(prong_vv, *iolcv, *ioll, adc_v);
      truthMatch_pid_ = truthMatch.pdg;
      truthMatch_tid_ = truthMatch.tid;
      truthMatch_purity_ = truthMatch.purity;
      truthMatch_completeness_ = truthMatch.completeness;
      truthMatch_nSimParts_ = (int)truthMatch.allMatches.size();
      for(unsigned int iM = 0; iM < truthMatch.allMatches.size(); ++iM){
        truthMatch_simPart_pid_[iM] = truthMatch.allMatches[iM].pdg;
        truthMatch_simPart_purity_[iM] = truthMatch.allMatches[iM].purity;
      }

      prongImg_plane0_nPix_ = (int)prong_vv[0].size();
      for(unsigned int iP = 0; iP < prong_vv[0].size(); ++iP){
        prongImg_plane0_row_[iP] = prong_vv[0][iP].row;
        prongImg_plane0_col_[iP] = prong_vv[0][iP].col;
        prongImg_plane0_val_[iP] = prong_vv[0][iP].val;
      }
      prongImg_plane1_nPix_ = (int)prong_vv[1].size();
      for(unsigned int iP = 0; iP < prong_vv[1].size(); ++iP){
        prongImg_plane1_row_[iP] = prong_vv[1][iP].row;
        prongImg_plane1_col_[iP] = prong_vv[1][iP].col;
        prongImg_plane1_val_[iP] = prong_vv[1][iP].val;
      }
      prongImg_plane2_nPix_ = (int)prong_vv[2].size();
      for(unsigned int iP = 0; iP < prong_vv[2].size(); ++iP){
        prongImg_plane2_row_[iP] = prong_vv[2][iP].row;
        prongImg_plane2_col_[iP] = prong_vv[2][iP].col;
        prongImg_plane2_val_[iP] = prong_vv[2][iP].val;
      }
      contextImg_plane0_nPix_ = (int)prong_vv[3].size();
      for(unsigned int iP = 0; iP < prong_vv[3].size(); ++iP){
        contextImg_plane0_row_[iP] = prong_vv[3][iP].row;
        contextImg_plane0_col_[iP] = prong_vv[3][iP].col;
        contextImg_plane0_val_[iP] = prong_vv[3][iP].val;
      }
      contextImg_plane1_nPix_ = (int)prong_vv[4].size();
      for(unsigned int iP = 0; iP < prong_vv[4].size(); ++iP){
        contextImg_plane1_row_[iP] = prong_vv[4][iP].row;
        contextImg_plane1_col_[iP] = prong_vv[4][iP].col;
        contextImg_plane1_val_[iP] = prong_vv[4][iP].val;
      }
      contextImg_plane2_nPix_ = (int)prong_vv[5].size();
      for(unsigned int iP = 0; iP < prong_vv[5].size(); ++iP){
        contextImg_plane2_row_[iP] = prong_vv[5][iP].row;
        contextImg_plane2_col_[iP] = prong_vv[5][iP].col;
        contextImg_plane2_val_[iP] = prong_vv[5][iP].val;
      }

      n_spacePoints_ = partMapPair.second.larflowProng.size();
      for(unsigned int iH = 0; iH < partMapPair.second.larflowProng.size(); ++iH){
        const auto& hit = partMapPair.second.larflowProng[iH];
        spacePoint_x_[iH] = hit[0];
        spacePoint_y_[iH] = hit[1];
        spacePoint_z_[iH] = hit[2];
        spacePoint_tick_[iH] = hit.tick;
        spacePoint_p0wire_[iH] = hit.targetwire[0];
        spacePoint_p1wire_[iH] = hit.targetwire[1];
        spacePoint_p2wire_[iH] = hit.targetwire[2];
      }

      cropPoint_x_ = partMapPair.second.cropPoint.X();
      cropPoint_y_ = partMapPair.second.cropPoint.Y();
      cropPoint_z_ = partMapPair.second.cropPoint.Z();

      std::cout << "filling ProngTree with larpid outputs, truth info, and images" << std::endl;
      tree_ -> Fill();

    }

  }

}


void LArPIDInterfaceTest::endJob() {
  delete iolcv;
  delete ioll;
}


DEFINE_ART_MODULE(LArPIDInterfaceTest)
